/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.example.browser;

import net.jini.admin.JoinAdmin;
import net.jini.admin.Administrable;
import net.jini.lookup.DiscoveryAdmin;
import com.sun.jini.admin.DestroyAdmin;
import net.jini.core.entry.Entry;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import com.sun.jini.lease.LeaseRenewalManager;
import com.sun.jini.lease.LeaseRenewalEvent;
import com.sun.jini.lease.LeaseListener;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.net.MalformedURLException;
import java.util.StringTokenizer;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.awt.Toolkit;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.DefaultListModel;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

/**
 * JoinAdministrator is a gui-based utility to add/modify
 * attributes, groups and lookup locators.
 * And also supports some well known admin iterfaces.
 * (DestroyAdmin, DiscoveryAdmin)
 *
 * <p>
 * current issues<br>
 * <ul>
 *   <li> can't operate(add/remove/modify) array elements in an entry.
 *   <li> does not support EntryBean.
 *   <li> field modification is not based on EditableTree
 * </ul>
 *
 * @version 0.2 06/24/98
 * @author Haruo Fukuda
 */
public class ServiceEditor extends JFrame {
  /**
   * @serial
   */
  private ServiceItem item;

  /**
   * @serial
   */
  private ServiceRegistrar registrar;

  /**
   * @serial
   */
  protected Object adminRoot = null;

  /**
   * @serial
   */
  protected JoinAdmin jadmin = null;

  /**
   * @serial
   */
  private ServiceTemplate stmpl;

  /**
   * @serial
   */
  private NotifyReceiver receiver;

  /**
   * @serial
   */
  private Lease elease = null;

  /**
   * @serial
   */
  private LeaseRenewalManager leaseMgr = new LeaseRenewalManager();

  /**
   * @serial
   */
  private AttributeTreePanel attrPanel;

  private final static int MINIMUM_WINDOW_WIDTH = 320;

  public ServiceEditor(ServiceItem item, ServiceRegistrar registrar){
    this(item, registrar, null);
  }

  public ServiceEditor(ServiceItem item, ServiceRegistrar registrar, Frame parent){
    super("ServiceItem Editor");
    
    this.item = item;
    this.registrar = registrar;
    try {
      adminRoot = ((Administrable) (item.service)).getAdmin();
      jadmin = (JoinAdmin) adminRoot;
    } catch(RemoteException e){
      // internal error
      e.printStackTrace();
    }

    // init main components
    attrPanel = new AttributeTreePanel();

    // setup notify
    try {
      stmpl = new ServiceTemplate(item.serviceID,
				  new Class[] { item.service.getClass() },
				  new Entry[] {});
      receiver = new NotifyReceiver();

      setupNotify();
    } catch (Exception e) {
      e.printStackTrace();
      cancelNotify();
    }

    // add menu and attr panel
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(new JoinMenuBar(), "North");
    getContentPane().add(attrPanel, "Center");

    validate();
    pack();
    setSize(((getSize().width < MINIMUM_WINDOW_WIDTH) ? MINIMUM_WINDOW_WIDTH : getSize().width),
	    getSize().height);

    // set frame location
    int xpos = 0;
    int ypos = 0;
    if(parent == null){
      // no parent frame is defined. set the center of screen
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension dialogSize = getPreferredSize();
      xpos = (screenSize.width - dialogSize.width)/2;
      ypos = (screenSize.height - dialogSize.height)/2;
    } else {
      // centerize to parent frame
      Rectangle bounds = parent.getBounds();
      Dimension dialogSize = getPreferredSize();
      xpos = bounds.x + (bounds.width - dialogSize.width)/ 2;
      ypos = bounds.y + (bounds.height - dialogSize.height)/2;
    }
    setLocation((xpos < 0) ? 0 : xpos,
		(ypos < 0) ? 0 : ypos);
  }

  protected void cancelNotify() {
    if(elease != null) {
      //System.out.println("[cancelNotify]");
      try {
	leaseMgr.cancel(elease);
      } catch(Exception e) {
	// ignore any exceptions
	e.printStackTrace();
      }
      elease = null;
    }
  }

  protected void setupNotify() {
    if(registrar != null) {
      //System.out.println("[setupNotify]");
      try {
	Thread.sleep(500);	// make sure that setting up notify is later than notify event.

	elease = registrar.notify(stmpl,
				  ServiceRegistrar.TRANSITION_MATCH_NOMATCH |
				  ServiceRegistrar.TRANSITION_NOMATCH_MATCH |
				  ServiceRegistrar.TRANSITION_MATCH_MATCH,
				  receiver,
				  null,
				  Lease.ANY).getLease();
	leaseMgr.renewUntil(elease, Lease.ANY, new LeaseNotify());
      } catch(Exception e) {
	e.printStackTrace();
      }
    }
  }

  private class NotifyReceiver extends UnicastRemoteObject implements RemoteEventListener {
    public NotifyReceiver() throws RemoteException {
      super();
    }

    public void notify(RemoteEvent ev) {
      System.out.println("[NotifyReceiver]: receive notification");
      attrPanel.receiveNotify();
    }
  }

  private static class LeaseNotify implements LeaseListener {
    public void notify(LeaseRenewalEvent ev) {
      if(ev.getException() != null)
	ev.getException().printStackTrace(System.out);
      else
	System.out.println("event lease failed to renew");
    }
  }

  class JoinMenuBar extends JMenuBar {
    public JoinMenuBar() {
      JMenuItem mitem;

      // "File" Menu
      JMenu fileMenu = (JMenu) add(new JMenu("File"));
      mitem = (JMenuItem) fileMenu.add(new JMenuItem("Show Info"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  Class[] infs = item.service.getClass().getInterfaces();
	  String[] msg = new String[3 + infs.length];
	  msg[0] = "ServiceID: " + item.serviceID;
	  msg[1] = "Service Instance: " + item.service.getClass().getName();
	  if(infs.length == 1)
	    msg[2] = "Implemented Interface:";
	  else
	    msg[2] = "Implemented Interfaces:";
	  for(int i = 0; i < infs.length; i++)
	    msg[3 + i] = "    " + infs[i].getName();

	  JOptionPane.showMessageDialog(ServiceEditor.this,
					msg,
					"ServiceItem Information",
					JOptionPane.INFORMATION_MESSAGE);
	}
      });
      mitem = (JMenuItem) fileMenu.add(new JMenuItem("Refresh"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  attrPanel.refreshPanel();
	}
      });
      mitem = (JMenuItem) fileMenu.add(new JMenuItem("Close"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  // cancel lease
	  cancelNotify();

	  // release resources and close all child frames
	  ServiceEditor.this.dispose();
	}
      });

      // "Edit" Menu
      JMenu editMenu = (JMenu) add(new JMenu("Edit"));
      mitem = (JMenuItem) editMenu.add(new JMenuItem("Add Attribute..."));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  attrPanel.addAttr();
	}
      });
      mitem = (JMenuItem) editMenu.add(new JMenuItem("Remove Attribute"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  attrPanel.removeAttr();
	}
      });

      // "Admin" Menu
      JMenu adminMenu = (JMenu) add(new JMenu("Admin"));

      // Group (JoinAdmin)
      mitem = (JMenuItem) adminMenu.add(new JMenuItem("Joining groups..."));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  new GroupLister("Joining Groups").showFrame();
	}
      });

      // Locator (JoinAdmin)
      mitem = (JMenuItem) adminMenu.add(new JMenuItem("Joining locators..."));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  new LocatorLister("Joining Locators").showFrame();
	}
      });

      // separator
      adminMenu.addSeparator();

      // Group (DiscoveryAdmin)
      mitem = (JMenuItem) adminMenu.add(new JMenuItem("Member groups..."));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  new MemberGroupLister("Member Groups").showFrame();
	}
      });
      if(! (adminRoot instanceof DiscoveryAdmin))
	mitem.setEnabled(false);

      // Unicastport (DiscoveryAdmin)
      mitem = (JMenuItem) adminMenu.add(new JMenuItem("Unicast port..."));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  try {
	    String[] msg = { "Current port is " + ((DiscoveryAdmin) adminRoot).getUnicastPort(),
			     "Input a new value" };
	    String result = JOptionPane.showInputDialog(ServiceEditor.this,
							msg,
							"Unicast Port",
							JOptionPane.QUESTION_MESSAGE);

	    if(result == null)
	      return;

	    try {
	      int port = Integer.parseInt(result);
	      ((DiscoveryAdmin) adminRoot).setUnicastPort(port);
	    } catch(NumberFormatException e){
	      JOptionPane.showMessageDialog(ServiceEditor.this,
					    result + " is not acceptable.",
					    "Error",
					    JOptionPane.ERROR_MESSAGE);
	    } catch(IOException e){
	      JOptionPane.showMessageDialog(ServiceEditor.this,
					    "Failed to set unicast port.",
					    "IOException",
					    JOptionPane.ERROR_MESSAGE);
	    }
	  } catch(RemoteException e){
	    JOptionPane.showMessageDialog(ServiceEditor.this,
					  "Failed to communicate with DiscoveryAdmin service.",
					  "IOException",
					  JOptionPane.ERROR_MESSAGE);
	  }
	}
      });
      if(! (adminRoot instanceof DiscoveryAdmin))
	mitem.setEnabled(false);

      // separator
      adminMenu.addSeparator();

      // DestroyAdmin
      mitem = (JMenuItem) adminMenu.add(new JMenuItem("Destroy"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  if(JOptionPane.showConfirmDialog(ServiceEditor.this,
					   "Are you sure to destroy this service?",
					   "Query",
					   JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
	    try {
	      ((DestroyAdmin) adminRoot).destroy();
	      // cancel lease
	      cancelNotify();

	      // release resources and close all child frames
	      ServiceEditor.this.dispose();
	    } catch(RemoteException e){
	      JOptionPane.showMessageDialog(ServiceEditor.this,
					    "Failed to communicate with DestroyAdmin service.",
					    "IOException",
					    JOptionPane.ERROR_MESSAGE);
	    }
	}
      });
      if(! (adminRoot instanceof DestroyAdmin))
	mitem.setEnabled(false);
    }
  }

  class AttributeTreePanel extends EntryTreePanel {

    public AttributeTreePanel() {
      super(true);

      tree.addMouseListener(new DoubleClicker(this));
      refreshPanel();
    }

    protected Entry[] getEntryArray() {
      try {
	return jadmin.getLookupAttributes();
      } catch(RemoteException e){
	e.printStackTrace();
      } catch(IllegalArgumentException iae1){
	iae1.printStackTrace();
      }
      return null;
    }

    protected void receiveNotify() {
      String[] msg = {"Attributes have been modified by another client or service provider itself.",
		      "Do you want refresh attributes ?"};
      int result = JOptionPane.showConfirmDialog(AttributeTreePanel.this,
						 msg,
						 "Query",
						 JOptionPane.YES_NO_OPTION);
	
      if(result == JOptionPane.YES_OPTION)
	refreshPanel();
    }

    public void editField(ObjectNode node) {
      //System.out.println("Editable ObjectNode");

      String result = JOptionPane.showInputDialog(this,
						  "Input a new value",
						  "Modify a field",
						  JOptionPane.QUESTION_MESSAGE);

      if(result == null){
	//System.out.println("[canceled]");
      } else {
	// Save current value as template
	Entry template = cloneEntry((Entry) node.getEntryTop());
	Object oldVal = null;

	if(result.length() == 0){
	  //System.out.println("[null specified]");
	  oldVal = node.setValue(null);
	} else {
	  //System.out.println("[value]: " + result);
	  oldVal = node.setValue(result);
	}
	// modifyAttribute
	Object msg = null;
	try {
	  node.setObjectRecursive();
	  Entry attr = (Entry) node.getEntryTop();
	  //Entry template = (Entry) generateTemplate(attr);
	  //System.out.println("[template attr]:\n" + template);
	  //System.out.println("[modified attr]:\n" + attr);

	  // cancel notify while adding an attribute
	  cancelNotify();

	  jadmin.modifyLookupAttributes(new Entry[] { template }, new Entry[] { attr });

	  // 
	  setupNotify();

	  // Redraw node value
	  model.nodeChanged(node);
	} catch(SecurityException e){
	  msg = new String[] { "Failed to modify an attribute (SecurityException).",
				 "Message: " + e.getMessage() };
	  e.printStackTrace();
	} catch(NoSuchFieldException e){
	  //msg = "[internal error]: failed introspection";
	  e.printStackTrace();
	} catch(IllegalAccessException e){
	  //msg = "[internal error]: failed introspection";
	  e.printStackTrace();
	} catch(RemoteException e){
	  msg = "Failed to modify an attribute (RemoteException).";
	  //System.out.println("Failed to modify attribute");
	  e.printStackTrace();
	} finally {
	  if(msg != null) {
	    // recover tree node
	    try {
	      node.setValue(oldVal);
	      node.setObjectRecursive();
	    } catch(Exception e){
	      // don't care
	    }
	    model.nodeChanged(node);
	    //model.nodeStructureChanged(node);

	    // show dialog
	    JOptionPane.showMessageDialog(AttributeTreePanel.this,
					  msg,
					  "Error",
					  JOptionPane.ERROR_MESSAGE);
	  }
	}
      }
    }

    public void addAttr() {
      //System.out.println("[addAttr]");

      String result = JOptionPane.showInputDialog(this,
						  "Input an addning class name",
						  "Add an attribute",
						  JOptionPane.QUESTION_MESSAGE);

      if(result == null || result.length() == 0){
	//System.out.println("[canceled]");
      } else {
	Object msg = null;
	try {
	  Class clazz = Class.forName(result);
	  Object attr = clazz.newInstance();

	  if(! (attr instanceof Entry)){
	    msg = new String[] {"This class does not implement Entry interface.", result};
	  } else if(attr instanceof net.jini.lookup.entry.ServiceControlled){
	    msg = new String[] {"This class implements ServiceControlled interface.", result};
	  } else {
	    // cancel notify while adding an attribute
	    cancelNotify();

	    jadmin.addLookupAttributes(new Entry[] { (Entry) attr });
	    // add node of this attribute
	    ObjectNode node = new ObjectNode(attr, true);
	    root.add(node);
	    recursiveObjectTree(node);

	    // 
	    setupNotify();

	    // refresh view
	    model.nodesWereInserted(root,
				    new int[] { model.getIndexOfChild(root, node) });
	  }
	} catch(ClassNotFoundException e){
	  msg = new String[] {"Can't find this class:", result};
	} catch(InstantiationException e){
	  msg = "InstantiationException was thrown.";
	} catch(IllegalAccessException e){
	  msg = "IllegalAccessException was thrown.";
	} catch(RemoteException e){
	  msg = "RemoteException was thrown.";
	} finally {
	  if(msg != null)
	    JOptionPane.showMessageDialog(AttributeTreePanel.this,
					  msg,
					  "Warning",
					  JOptionPane.WARNING_MESSAGE);
	}
      }
    }

    public void removeAttr() {
      //System.out.println("[removeAttr]");

      ObjectNode node = (ObjectNode) tree.getLastSelectedPathComponent();
      if(node == null){
	JOptionPane.showMessageDialog(AttributeTreePanel.this,
				      "Select an attribute folder to remove.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	
	return;
      } else if(! node.isControllable()){
	JOptionPane.showMessageDialog(AttributeTreePanel.this,
				      "This attribute is under service provider's controll.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	
	return;
      } else if(! node.isEntryTop()){
	JOptionPane.showMessageDialog(AttributeTreePanel.this,
				      "Select a top of attribute folder.",
				      "Warning",
				      JOptionPane.WARNING_MESSAGE);
	
	return;
      }

      Entry target = (Entry) node.getObject();
      int result = JOptionPane.showConfirmDialog(AttributeTreePanel.this,
						 new String[] {"Remove attrbiute:",
								 target.toString() },
						 "Query",
						 JOptionPane.YES_NO_OPTION);
	
      if(result == JOptionPane.YES_OPTION){
	// Remote Attribute
	Object msg = null;
	try {
	  //System.out.println("[target attr]:\n" + target);

	  // cancel notify while adding an attribute
	  cancelNotify();

	  jadmin.modifyLookupAttributes(new Entry[] { target }, new Entry[] { null });

	  // 
	  setupNotify();

	  int index = root.getIndex(node);
	  root.remove(node);
	  model.nodesWereRemoved(root, new int[] {index}, new Object[] {node});
	  node = null;
	} catch(SecurityException e){
	  msg = new String[] { "Failed to remove an attribute (SecurityException).",
				 "Message: " + e.getMessage() };
	  e.printStackTrace();
	} catch(RemoteException e){
	  msg = "Failed to remove an attribute (RemoteException).";
	  //System.out.println("Failed to remove attribute");
	  e.printStackTrace();
	} finally {
	  if(msg != null) {
	    // show dialog
	    JOptionPane.showMessageDialog(AttributeTreePanel.this,
					  msg,
					  "Error",
					  JOptionPane.ERROR_MESSAGE);
	  }
	}
      }
    }

    private Entry cloneEntry(Entry attr) {
      try {
	Class realClass = attr.getClass();
	Entry template = (Entry) realClass.newInstance();

	Field[] f = realClass.getFields();
	for(int i = 0; i < f.length; i++) {
	  if(! usableField(f[i]))
	    continue;
	  f[i].set(template, f[i].get(attr));
	}

	return template;
      } catch(Exception e){
	//System.out.println("Failed to create clone of an entry.");
	e.printStackTrace();
      }
      return null;
    }

    // from EntryRep
    private boolean usableField(Field field) {
      Class desc = field.getDeclaringClass();

      if(desc.isPrimitive()) {
	throw new IllegalArgumentException("primitive types not allowed in an Entry");
      }

      // skip anything that isn't a public per-object mutable field
      int mods = field.getModifiers();
      return (0 == (mods & (Modifier.TRANSIENT | Modifier.STATIC | Modifier.FINAL)));
    }

    private Entry generateTemplate(Entry attr) {
      try {
	Class realClass = attr.getClass();
	Entry template = (Entry) realClass.newInstance();

	Field[] f = realClass.getFields();
	for(int i = 0; i < f.length; i++)
	  f[i].set(template, null);

	return template;
      } catch(Exception e){
	e.printStackTrace();
      }
      return null;
    }

    class DoubleClicker extends MouseAdapter {
      AttributeTreePanel parent;

      public DoubleClicker(AttributeTreePanel parent){
	this.parent = parent;
      }

      public void mouseClicked(MouseEvent ev){
	if(ev.getClickCount() >= 2){
	  JTree tree = (JTree) ev.getSource();
	  TreePath path = tree.getPathForLocation(ev.getX(), ev.getY());
	  if(path == null)
	    return;
	  ObjectNode node = (ObjectNode) path.getLastPathComponent();

	  //System.out.println("Double click on " + node);
	  if(node.isLeaf()){
	    if(! node.isControllable()){
	      JOptionPane.showMessageDialog(AttributeTreePanel.this,
					    "This attribute is under service provider's controll.",
					    "Warning",
					    JOptionPane.WARNING_MESSAGE);
	    } else if(node.isEditable()){
	      parent.editField(node);
	    } else {
	      JOptionPane.showMessageDialog(AttributeTreePanel.this,
					    "This field is not editable.",
					    "Warning",
					    JOptionPane.WARNING_MESSAGE);
	    }
	  }

	  tree.scrollPathToVisible(path);
	}
      }
    }
  }

  abstract class ListerFrame extends JFrame {

    private JList listBox;
    private JScrollPane scrollPane;
    protected DefaultListModel model = new DefaultListModel();
    private DefaultListModel dummyModel = new DefaultListModel();	// to keep away from Swing's bug

    private JButton addButton;
    private JButton removeButton;
    private JButton closeButton;

    public ListerFrame(String title) {
      super(title);

      getContentPane().setLayout(new BorderLayout());

      // create the initial list
      listBox = new JList(model);
      listBox.setFixedCellHeight(20);
      scrollPane = new JScrollPane(listBox);
      getContentPane().add(scrollPane, "Center");
      //resetListModel();

      // Create the controls 
      JPanel buttonPanel = new JPanel();
      addButton = new JButton("Add");
      addButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  String result = JOptionPane.showInputDialog(ListerFrame.this, getAddMessage());

	  if(result != null){
	    StringTokenizer st = new StringTokenizer(result);
	    String[] tokens = new String[st.countTokens()];
	    for(int i = 0; i < tokens.length; i++)
	      tokens[i] = st.nextToken().trim();

	    addItems(tokens);
	    resetListModel();
	    scrollPane.validate();
	  }
	}
      });
      buttonPanel.add(addButton);

      removeButton = new JButton("Remove");
      removeButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  Object[] selected = listBox.getSelectedValues();

	  if(selected == null || selected.length == 0){
	    // no items are selected
	    JOptionPane.showMessageDialog(ListerFrame.this,
					  "No items are selected",
					  "Warning",
					  JOptionPane.WARNING_MESSAGE);
	    return;
	  }

	  int result = JOptionPane.showConfirmDialog(ListerFrame.this,
						     getRemoveMessage(selected),
						     "Query",
						     JOptionPane.YES_NO_OPTION);
	
	  if(result == JOptionPane.YES_OPTION){
	    removeItems(selected);
	    resetListModel();
	    scrollPane.validate();
	  }
	}
      });
      buttonPanel.add(removeButton);

      closeButton = new JButton("Close");
      closeButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  setVisible(false);
	}
      });
      buttonPanel.add(closeButton);
      getContentPane().add(buttonPanel, "South");

      pack();
    }

    public void showFrame() {
      // init list data
      resetListModel();

      // centerize to parent frame
      Rectangle bounds = ServiceEditor.this.getBounds();
      Dimension dialogSize = getPreferredSize();

      //System.out.println("Parent's bounds=" + bounds);
      //System.out.println("My preferred size=" + dialogSize);

      setLocation(bounds.x + (bounds.width - dialogSize.width)/ 2,
		  bounds.y + (bounds.height - dialogSize.height)/2);

      setVisible(true);
    }

    private void resetListModel() {
      //listBox.setModel(null);
      listBox.setModel(dummyModel);	// to keep away from NullException (Swing's bug)

      model.removeAllElements();
      initListModel();

      listBox.setModel(model);
      listBox.clearSelection();
      listBox.ensureIndexIsVisible(0);
      listBox.repaint();
      listBox.revalidate();
    }

    protected abstract void initListModel();

    protected abstract String getAddMessage();

    protected abstract String getRemoveMessage(Object[] items);

    protected abstract void addItems(String[] items);

    protected abstract void removeItems(Object[] items);
  }

  class GroupLister extends ListerFrame {

    public GroupLister(String title) {
      super(title);
    }

    protected void initListModel() {
      try {
	String[] groups = jadmin.getLookupGroups();
	for(int i = 0; i < groups.length; i++) {
	  model.addElement(new GroupItem(groups[i]));
	}
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }

    protected String getAddMessage() {
      return "Enter adding group(s)";
    }

    protected String getRemoveMessage(Object[] items) {
      StringBuffer msg = new StringBuffer();
      if(items.length > 1)
	msg.append("Remove these groups : ");
      else
	msg.append("Remove a group : ");
      for(int i = 0; i < items.length; i++) {
	if(i != 0)
	  msg.append(", ");
	msg.append(((GroupItem) items[i]).toString());
      }
      return msg.toString();
    }

    protected void addItems(String[] items) {
      // check "public"
      String[] grps = new String[items.length];
      for(int i = 0; i < items.length; i++)
	grps[i] = new GroupItem(items[i]).group;

      try {
	jadmin.addLookupGroups(grps);
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }

    protected void removeItems(Object[] items) {
      String[] grps = new String[items.length];
      for(int i = 0; i < items.length; i++)
	grps[i] = ((GroupItem) items[i]).group;

      try {
	jadmin.removeLookupGroups(grps);
      } catch(RemoteException e){
      }
    }
  }

  class MemberGroupLister extends ListerFrame {

    public MemberGroupLister(String title) {
      super(title);
    }

    protected void initListModel() {
      try {
	String[] groups = ((DiscoveryAdmin) adminRoot).getMemberGroups();
	for(int i = 0; i < groups.length; i++) {
	  model.addElement(new GroupItem(groups[i]));
	}
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }

    protected String getAddMessage() {
      return "Enter adding group(s)";
    }

    protected String getRemoveMessage(Object[] items) {
      StringBuffer msg = new StringBuffer();
      if(items.length > 1)
	msg.append("Remove these groups : ");
      else
	msg.append("Remove a group : ");
      for(int i = 0; i < items.length; i++) {
	if(i != 0)
	  msg.append(", ");
	msg.append(((GroupItem) items[i]).toString());
      }
      return msg.toString();
    }

    protected void addItems(String[] items) {
      // check "public"
      String[] grps = new String[items.length];
      for(int i = 0; i < items.length; i++)
	grps[i] = new GroupItem(items[i]).group;

      try {
	((DiscoveryAdmin) adminRoot).addMemberGroups(grps);
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }

    protected void removeItems(Object[] items) {
      String[] grps = new String[items.length];
      for(int i = 0; i < items.length; i++)
	grps[i] = ((GroupItem) items[i]).group;

      try {
	((DiscoveryAdmin) adminRoot).removeMemberGroups(grps);
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }
  }

  class GroupItem {
    public String group;

    public GroupItem(String group) {
      if(group.equals("public"))
	this.group = "";
      else
	this.group = group;
    }

    public String toString() {
      if("".equals(group))
	return "public";
      else
	return group;
    }
  }

  class LocatorLister extends ListerFrame {

    public LocatorLister(String title) {
      super(title);
    }

    protected void initListModel() {
      try {
	LookupLocator[] locators = jadmin.getLookupLocators();
	for(int i = 0; i < locators.length; i++) {
	  model.addElement(locators[i]);
	}
      } catch(RemoteException e){
	e.printStackTrace();
      } catch(Exception e){
	e.printStackTrace();
      }
    }

    protected String getAddMessage() {
      return "Enter a new locator's URL";
    }

    protected String getRemoveMessage(Object[] items) {
      StringBuffer msg = new StringBuffer();
      if(items.length > 1)
	msg.append("Remove these locators : ");
      else
	msg.append("Remove a locator : ");
      for(int i = 0; i < items.length; i++) {
	if(i != 0)
	  msg.append(", ");
	msg.append(items[i].toString());
      }
      return msg.toString();
    }

    protected void addItems(String[] items) {
      // check "public"
      LookupLocator[] locs = new LookupLocator[items.length];
      for(int i = 0; i < items.length; i++) {
	try {
	  locs[i] = new LookupLocator(items[i]);
	} catch(MalformedURLException e){
	  JOptionPane.showMessageDialog(LocatorLister.this,
					"Malformed URL: " + items[i],
					"Warning",
					JOptionPane.WARNING_MESSAGE);
	  return;
	}
      }

      try {
	jadmin.addLookupLocators(locs);
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }

    protected void removeItems(Object[] items) {
      LookupLocator[] locs = new LookupLocator[items.length];
      for(int i = 0; i < items.length; i++)
	locs[i] = (LookupLocator) items[i];

      try {
	jadmin.removeLookupLocators(locs);
      } catch(RemoteException e){
	e.printStackTrace();
      }
    }
  }
}
