/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.example.service;

import java.io.*;
import java.lang.*;
import com.sun.jini.thread.StreamPlugThread;

public class RunSpace {
    
    private Process p1;
    private Runtime rt;
    private String  persistentDir;
    private String  runCmd;

    
    public RunSpace( String registryType,
		     String registryPort,
		     String jarTop,
		     String outriggerServer,
		     String transientOutriggerServer,
		     String urlTop,
		     String outriggerClient,
		     String policyFile,
		     String persistentDir,
		     String lookupGroups,
		     String spaceType,
		     String spaceName ) {



	if ( registryType.equals("No Services") ) {
  	  System.err.println(" You need to select a registry service");
	  return;
        }

	
	String newServer = "";

	if ( spaceType.equals("Transient") )
	    newServer = transientOutriggerServer;
	
	if ( spaceType.equals("FrontEnd") )
	    newServer = outriggerServer;

	if ( ! jarTop.equals("") )
	    newServer = jarTop + File.separator + newServer;
	
	String newClient;
	if ( urlTop.equals("") )
	    newClient = outriggerClient;
	else
	    newClient = urlTop + "/" + outriggerClient;

	File test = new File ( newServer );
   	if ( ! test.isFile() ) {
	    System.err.println("Outrigger can't be found at " + newServer);
	   return;
        }


	runCmd = "java -jar ";
	runCmd = runCmd + "-Djava.security.policy=" + policyFile;

        if ( registryType.equals("RMI Registry")) {
	    lookupGroups="none";
	    runCmd = runCmd + " -Dcom.sun.jini.use.registry=yes ";
	    if ( ! registryPort.equals("") )
		runCmd = runCmd + 
		    " -Dcom.sun.jini.rmiregistryport=" + 
		    registryPort;
        }

	if ( spaceType.equals("Transient")) {
	    System.err.println("TransientSpace");
	    runCmd = runCmd + 
		" -Djava.rmi.server.codebase=" + newClient               + 
		" " + " -Dcom.sun.jini.outrigger.spaceName=" + spaceName + 
		" " + newServer + " " + lookupGroups;
	}


	if ( spaceType.equals("FrontEnd")) {
	   System.err.println("FrontEndSpace");
	   runCmd = runCmd + 
	       " -Dcom.sun.jini.outrigger.spaceName=" + spaceName +  
	       " " + newServer     +
	       " " + newClient     +  
	       " " + policyFile    + 
	       " " + persistentDir + 
	       " " + lookupGroups;
        }

        
        try {
            Runtime rt = Runtime.getRuntime();
	    

    	   System.err.println("Running " + runCmd );
	
	   p1 = rt.exec(runCmd);
	    
           StreamPlugThread.plugTogether(p1.getInputStream(), System.out);
	   StreamPlugThread.plugTogether(p1.getErrorStream(), System.err);


	}//try

	catch (IOException e) {
	    System.err.println("exec error:" + e);
	}
    }//RunSpace

    public void destroy() {
	System.err.println(" Killing Spaces");
	p1.destroy();
    }//destroy
}//RunSpace


