/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.example.service;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import java.awt.GridLayout;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Font;
//import com.sun.jini.example.space.RunActivation;
//import com.sun.jini.example.space.RunRMIRegistry;
//import com.sun.jini.example.space.RunTxnManager;
//import com.sun.jini.example.space.RunLookup;
//import com.sun.jini.example.space.RunHttpd;
//import com.sun.jini.example.space.RunSpace;

public class RunitGUI
{
    
    private boolean    rmiBinderFlag =  false;
    private String     binderName;
    private JPanel     panel;
    private GridLayout classesGUILayout;

    private JButton    startRMID,stopRMID;
    private JButton    startRegistryService,stopRegistryService;
    private JButton    startTxnManager,stopTxnManager;
    private JButton    startHttpd,stopHttpd;
    private JButton    startSpace,stopSpace;

    private ClassesGUI    classesGUIRef;
    private HttpdGUI      httpdGUIRef;
    private ActivationGUI activationGUIRef;
    private RegistryGUI   registryGUIRef;
    private PolicyGUI     policyGUIRef;
    private TxnGUI        txnGUIRef;
    private SpacesGUI      spacesGUIRef;

    private RunHttpd       httpdProcess;
    private RunActivation  rmidProcess;
    private RunRMIRegistry rmiRegistryProcess;
    private RunLookup      lookupProcess;
    private RunTxnManager  txnManagerProcess;
    private RunSpace       spaceProcess;

    
    public RunitGUI( JPanel panel, String text,
		     ClassesGUI  classesGUI,
		     HttpdGUI httpdGUI,
		     ActivationGUI activationGUI,
		     RegistryGUI registryGUI,
		     PolicyGUI   policyGUI,
		     TxnGUI      txnGUI,
		     SpacesGUI    spacesGUI) {

	//Save references to the GUI components for later
	//retreival of panel sub-components
	classesGUIRef    = classesGUI;
	httpdGUIRef      = httpdGUI;
	activationGUIRef = activationGUI;
	registryGUIRef   = registryGUI;
	policyGUIRef     = policyGUI;
	txnGUIRef        = txnGUI;
        spacesGUIRef      = spacesGUI;

	//Setup the Look and Feel for this Panel
	classesGUILayout = (new GridLayout(0, 2));
	classesGUILayout.setHgap(3);
	classesGUILayout.setVgap(3);

	panel.setLayout(classesGUILayout);
	//panel.setFont(new Font("Dialog", Font.PLAIN, 24));

	//Populate the GUI Piecces

	startHttpd = new JButton("Start Httpd Daemon");
	stopHttpd  = new JButton("Stop Httpd Daemon");

	startRMID = new JButton("Start RMID");
	stopRMID  = new JButton("stop RMID");

	startRegistryService = new JButton("Start NO SERVICES");
	stopRegistryService  = new JButton("Stop NO SERVICES");

	startTxnManager = new JButton("Start Txn Manager");
	stopTxnManager  = new JButton("Stop Txn Manager");

	startSpace = new JButton("Start Space");
	stopSpace  = new JButton("Stop Space");

	//Add Callbacks for Mouse Events
	SymMouse aSymMouse = new SymMouse();
	startHttpd.addMouseListener(aSymMouse);
	stopHttpd.addMouseListener(aSymMouse);
	startRMID.addMouseListener(aSymMouse);
	stopRMID.addMouseListener(aSymMouse);
	startRegistryService.addMouseListener(aSymMouse);
	stopRegistryService.addMouseListener(aSymMouse);
	startTxnManager.addMouseListener(aSymMouse);
	stopTxnManager.addMouseListener(aSymMouse);
	startSpace.addMouseListener(aSymMouse);
	stopSpace.addMouseListener(aSymMouse);

	//Populate the panel
	panel.add(new JLabel(" "));
	panel.add(new JLabel(" "));

	panel.add(startHttpd);
	panel.add(stopHttpd);

	panel.add(startRMID);
	panel.add(stopRMID);

	panel.add(startRegistryService);
	panel.add(stopRegistryService);

	panel.add(startTxnManager);
	panel.add(stopTxnManager);

	panel.add(startSpace);
	panel.add(stopSpace);

	panel.add(new JLabel(" "));
	panel.add(new JLabel(" "));
    }//end-constructor
    
    class SymMouse extends java.awt.event.MouseAdapter {
	
	public void mouseClicked(java.awt.event.MouseEvent event) {
	    
	    Object object = event.getSource();

	    if (object == startRMID) {
		System.err.println("Starting to Run RMID");
		rmidProcess = new 
			RunActivation( activationGUIRef.getrmidOptions()) ;
		System.err.println("RMID is up");
	    }

	    if (object == stopRMID) {
		// need to add code that will check to see if we
		// actually started rmid first
		if (rmidProcess == null) {
		    System.err.println("RMID is not running");
		} else {
		    rmidProcess.destroy();
		    System.err.println("RMID is down");
		}
	    }

	    if (object == startHttpd) {
		System.err.println("Starting to run httpd");
		httpdProcess = new RunHttpd( httpdGUIRef.getHttpdPort(),
				httpdGUIRef.getHttpdDoc(),
				httpdGUIRef.getHttpdTree(),
				httpdGUIRef.getHttpdVerbose(),
				classesGUIRef.getJarTop(),
				classesGUIRef.getTools());
		System.err.println("Web server is up");
            }

	    if (object == stopHttpd) {
		if (httpdProcess == null) {
		    System.err.println("httpd server is not running");
		} else {
		    httpdProcess.destroy();
		    System.err.println("httpd server is down");
		}
	    }

	    if (object == startRegistryService) {
		if (registryGUIRef.getRMIRegistryState()) {
		    rmiBinderFlag = true;
		    rmiRegistryProcess = new RunRMIRegistry();
		    System.err.println("RMID is up");
		}
	    }

	    if (object == startRegistryService) {
		if (registryGUIRef.getLookupState()) {
		    rmiBinderFlag = false;
		    lookupProcess = new RunLookup
			(classesGUIRef.getJarTop(),
			 classesGUIRef.getReggieServer(),
			 classesGUIRef.getURLTop(),
			 classesGUIRef.getReggieClient(),
			 policyGUIRef.getLookupPath(),
			 registryGUIRef.getLookupLog(),
			 registryGUIRef.getLookupGroups() );

		    System.err.println("Lookup is up");
		}
	    }

	    if (object == stopRegistryService) {
		if (registryGUIRef.getRMIRegistryState()) {
		    if (rmiRegistryProcess == null) {
			System.err.println("RMI Registry is not running");
		    } else {
			rmiRegistryProcess.destroy();
			System.err.println("RMI Registry is down");
		    }
		}
	    }

	    if (object == stopRegistryService) {
		if (registryGUIRef.getLookupState()) {
		    if (lookupProcess == null) {
			System.err.println("RMI Registry is not running");
		    } else {
			lookupProcess.destroy();
			System.err.println("RMI Registry is down");
		    }
		}
	    }

	    if (object == startTxnManager) {
		if (rmiBinderFlag == true) {
		    binderName = "rmi-registry";
		} else {
		    binderName = "jini-lookup";
		}

		txnManagerProcess = 
		    new RunTxnManager(binderName,
				      registryGUIRef.getRegistryPort(),
				      classesGUIRef.getJarTop(),
				      classesGUIRef.getMahaloServer(),
				      classesGUIRef.getURLTop(),
				      classesGUIRef.getMahaloClient(),
				      policyGUIRef.gettxnPath(),
				      txnGUIRef.getTxnLogDir(),
				      registryGUIRef.getLookupGroups(),
				      txnGUIRef.getTxnOptions());
						     
	    }//end-ifstartTxnManager
		
	    if (object == stopTxnManager) {
		if (txnManagerProcess == null) {
		    System.err.println("Txn Manager is not running");
		} else {
		    txnManagerProcess.destroy();
		    System.err.println("Txn Manager is down");
		}
	    }

	    if (object == startSpace) {
		spaceProcess =
		  new RunSpace(registryGUIRef.getCurrentRegistryType(),
				registryGUIRef.getRegistryPort(),
				classesGUIRef.getJarTop(),
				classesGUIRef.getOutriggerServer(),
			        classesGUIRef.getTransientOutriggerServer(),
				classesGUIRef.getURLTop(),
				classesGUIRef.getOutriggerClient(),
				policyGUIRef.getSpacePath(),
				spacesGUIRef.getPersistentDir(),
				registryGUIRef.getLookupGroups(),
				spacesGUIRef.getSpaceType().toString(),
				spacesGUIRef.getSpaceName());
           }

	    if (object == stopSpace) {
		if (spaceProcess == null) {
		    System.err.println("Spaces Server is not running");
		} else {
		    spaceProcess.destroy();
		    System.err.println("Spaces Server is down");
		}
	    }
				
	}//end-mouseClicked
    }//end-SymMouse

    public void setRegistryText(String text) {
	startRegistryService.setText("Start " + text);
	stopRegistryService.setText("Stop " + text);
    }
}//end-class



