/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.fiddler;

import java.io.Serializable;

/**
 * When a program starts the activatable version of the Fiddler implementation
 * of the lookup discovery service, this class is the data structure that 
 * must be used to pass (in marshalled form) the values of the required
 * and optional arguments expected by that implementation.
 */
public class FiddlerStartParams implements Serializable
{
    private static final long serialVersionUID = -1629942821408694014L;
    /**
     * The name of the directory to which persistent state will be logged.
     * @serial
     */
    final String logDirname;
    /** 
     * Groups whose members are the lookup services to initially join.
     * @serial
     */
    final String[] groups;
    /** 
     * Either the locators or the specific lookup services to discover and
     * join, or the names of the hosts on which those lookup services are
     * running.
     * @serial
     */
    final Object[] locators;

    /**
     * Constructs a new instance of FiddlerStartParams.
     *
     * @param logDirname the name of the directory to which persistent state
     *                   will be logged
     * @param groups     the names of the groups whose members are the lookup
     *                   services to initially join
     * @param locators   the names of the hosts on which the specific lookup
     *                   services to initially discover and join are running 
     *                   (these host names will eventually used to build
     *                   corresponding net.jini.core.discovery.LookupLocator
     *                   instances)
     */
    public FiddlerStartParams(String logDirname,
                              String[] groups,
                              Object[] locators)
    {
	this.logDirname = logDirname;
	this.groups     = groups;
	this.locators   = locators;
    }
}
