/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.fiddler;

import net.jini.core.discovery.LookupLocator;
import java.util.Properties;

/**
 * This class serves as a data structure, (and associated data accessor
 * methods), which can be used to store and return the values of the arguments
 * extracted from the command line when starting a client or service.
 */
class ParsedArgs {
    private String          codebase;
    private String          policyFile;
    private String          logDir;
    private String[]        groups;
    private LookupLocator[] locators;
    private String          javaProgram;
    private Properties      properties;
    private String[]        options;

    /* Constructs an instance of this data structure with the appropriate
     * values extracted from the command line of the client or service being
     * started.
     */
    public ParsedArgs(String          codebase,
                      String          policyFile,
                      String          logDir,
                      String[]        groups,
                      LookupLocator[] locators,
                      String          javaProgram,
                      Properties      properties,
                      String[]        options)
    {
        this.codebase    = codebase;
        this.policyFile  = policyFile;
        this.logDir      = logDir;
        this.groups      = groups;
        this.locators    = locators;
        this.javaProgram = javaProgram;
        this.properties  = properties;
        this.options     = options;
    }//end constructor

    public String getCodebase() {
        return codebase;
    }

    public String getPolicyFile() {
        return policyFile;
    }

    public String getLogDir() {
        return logDir;
    }

    public String[] getGroups() {
        return groups;
    }

    public LookupLocator[] getLocators() {
        return locators;
    }

    public String getJavaProgram() {
        return javaProgram;
    }

    public Properties getProperties() {
        return properties;
    }

    public String[] getOptions() {
        return options;
    }
}//end class ParsedArgs
