/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */

package com.sun.jini.lease;

import net.jini.core.lease.*;

/**
 * @deprecated This class was experimental; as of 1.1Alpha, it has been
 *	replaced by {@link net.jini.lease.LeaseRenewalEvent}. This class  
 *	was needed by <tt>net.jini.lease.LeaseRenewalManager</tt>. This  
 *	class will be removed in a future release, so please update your
 *      code, and refer to the javadoc for 
 *      <tt>net.jini.lease.LeaseRenewalEvent</tt> for details on 
 *      <code>LeaseRenewalEvent</code>.
 * 
 * @see net.jini.lease.LeaseRenewalEvent
 * @see net.jini.lease.LeaseRenewalManager
 */
public class LeaseRenewalEvent extends java.util.EventObject {
    /**
     * The failed lease.
     *
     * @serial
     */
    private Lease lease;
    /**
     * The desired expiration of the failed lease.
     *
     * @serial
     */
    private long expiration;
    /**
     * The exception that caused the failure, if any.
     *
     * @serial
     */
    private Exception ex;

    /**
     * Construct an event.
     *
     * @param source the lease renewal manager that generates the event
     * @param lease the lease that failed to renew
     * @param expiration the desired absolute expiration for the lease
     * @param ex the exception, if any
     */
    public LeaseRenewalEvent(LeaseRenewalManager source,
			     Lease lease,
			     long expiration,
			     Exception ex) 
    {
	super(source);
	this.lease = lease;
	this.expiration = expiration;
	this.ex = ex;
    }

    private static final long serialVersionUID = -7536467680016825016L;

    /** Returns the lease that failed to renew. */
    public Lease getLease() {
	return lease;

    }


    /** Returns the desired absolute expiration for the lease. */
    public long getExpiration() {
	return expiration;
    }


    /**
     * Returns the exception, if any.  If the lease was denied, then
     * this will be a LeaseDeniedException.  If the lease expired,
     * this will be a LeaseUnknownException, a RemoteException, or null.
     */
    public Exception getException() {
	return ex;
    }
}
