/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.lease.landlord;

/**
 * <code>LeaseManager</code> provides an interface for tracking lease
 * status.  Some implementations of <code>LeasePolicy</code>
 * (e.g. <code>LeaseDurationPolicy</code>) allow registration of a
 * <code>LeaseManager</code> that will be notified when a new Lease is
 * created or a Lease's duration changes.
 * <p>
 * The <code>LeaseManager</code> interface has methods indicating when
 * new leases have been granted and when existing leases have been
 * renewed, it does not have a method to indicate when a lease has
 * been cancelled.  Because of this, <code>LeaseManager</code>
 * implementations need to be careful not to leak memory/resources if
 * leases are cancelled before they expire.  In particular, they
 * should not hold on to <code>LeasedResource</code>s directly since
 * this would prevent garbage collection of resources when their
 * leases are cancelled before they expire.
 *
 * @see LeaseDurationPolicy
 * @see LeasedResource
 */
public interface LeaseManager {
    /**
     * Notifies the manager of a new lease being created.
     * @param resource   The resource associated with the new Lease.
     * @param duration   The duration of the lease
     */
    public void register(LeasedResource resource, long duration);

    /**
     * Notifies the manager of a lease being renewed.
     * @param resource      The resource associated with the new Lease.
     * @param duration      The duration of the lease
     * @param oldExpiration The expiration the resource had before it 
     *                      was renewed
     */
    public void renewed(LeasedResource resource, long duration, 
			long oldExpiration);
}



