/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahalo;

import java.io.IOException;
import com.sun.jini.mahout.ExecUtilBase;

/**
 * Utility class for starting Mahalo Transaction Managers
 */
public class MahaloExecUtil extends ExecUtilBase {

    /** 
     * Change codebase, classpath, and/or policyFilePath the
     * appropriate <code>com.sun.jini.exec.mahalo.*</code> property has
     * been set
     */
    private void constructorUtil() {
	final String mahaloCodebase = 
	    System.getProperty("com.sun.jini.exec.mahalo.codebase");

	if (codebase == null && mahaloCodebase == null)
	    throw new IllegalStateException("MahaloExecUtil:Must specify " +
	         "a codebase using com.sun.jini.exec.mahalo.codebase or " +
		 "com.sun.jini.exec.codebase");
	
	if (mahaloCodebase != null) codebase = mahaloCodebase;


	final String mahaloClasspath = 
	    System.getProperty("com.sun.jini.exec.mahalo.class.path");

	if (classpath == null && mahaloClasspath == null)
	    throw new IllegalStateException("MahaloExecUtil:Must specify " +
	         "a classpath using com.sun.jini.exec.mahalo.class.path or " +
		 "com.sun.jini.exec.class.path");
	
	if (mahaloClasspath != null) classpath = mahaloClasspath;



	final String mahaloPolicy = 
	    System.getProperty("com.sun.jini.exec.mahalo.policy");

	if (policyFilePath == null && mahaloPolicy == null)
	    throw new IllegalStateException("MahaloExecUtil:Must specify " +
	         "a policy file using com.sun.jini.exec.mahalo.policy or " +
		 "com.sun.jini.exec.policy");
	
	if (mahaloPolicy != null) policyFilePath = mahaloPolicy;
    }

    /**
     * Append <code>cmd</code> with approprate string to set the
     * transaction manager's name to <code>name</code>.
     * <code>name</code> may be <code>null</code>.
     */
    private void addName(String name) {
	if (name == null) return;

	cmd += "-Dcom.sun.jini.mahalo.managerName=" + name + " ";
    }

    /**
     * Set <code>cmd</code>
     * @param useRegistry If <code>true</code> setup the service to bind into
     *                    RMI registry.  Otherwise use Jini lookup
     * @param port        Port to find the RMI registry on. Ignored if 
     *                    <code>useRegistry</code> is <code>false</code>.
     * @param groups      Lookup groups the service should bind into. Ignored
     *                    if <code>useRegistry</code> is <code>true</code>.
     * @param name        Name to register the service under.
     */
    private void buildCmd(boolean useRegistry, int port, String groups,
			  String name)
    {
	cmd += "-Djava.security.policy=" + policyFilePath  + " ";

	if (useRegistry) {
	    cmd += "-Dcom.sun.jini.use.registry=yes ";
	    if (port >= 0)
		cmd += "-Dcom.sun.jini.rmiRegistryPort=" + port + " ";
	}

	cmd += "-Djava.class.path=" + classpath + " ";

	addName(name);

	cmd += "com.sun.jini.mahalo.TxnManagerImpl " + codebase + " " + 
	    policyFilePath + " " + persistencePath + " " + groups;
    }

    /**
     * Build an MahaloExecUtil for a Transaction Manager that will bind itself
     * into the registry
     *
     * @param port   Registry port to bind to 
     * @param name   The transaction manager's name
     */
    public MahaloExecUtil(int port, String name) throws IOException 
    {
	super("Mahalo");
	constructorUtil();
	buildCmd(true, port, "none", name);
    }

    /**
     * Build an MahaloExecUtil for a Transaction Manager that will bind itself
     * into a lookup service
     *
     * @param groups Lookup group list
     * @param name   The transaction manager's name
     */
    public MahaloExecUtil(String groups, String name) throws IOException 
    {
	super("Mahalo");
	constructorUtil();
	buildCmd(false, -1, groups, name);
    }
}



