/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahalo;

import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;

import com.sun.jini.mahout.StorableObject;

import java.io.Serializable;
import java.rmi.RemoteException;


class ParticipantHandle implements Serializable, TransactionConstants {
    static final long serialVersionUID = -1776073824495304317L;

    /**
     * @serial
     */
    private StorableObject storedpart;

    /**
     * @serial
     */
    private long crashcount = 0;

    /**
     * @serial
     */
    private int prepstate;

    private static final boolean DEBUG = false;

    /**
     * Create a new handle, calculating the hash for the object.
     */
    ParticipantHandle(TransactionParticipant part)
	throws RemoteException {
	this(part, 0);
    }

    /**
     * Create a new node that is equivalent to that node
     */
    ParticipantHandle(TransactionParticipant part, long crashcount) 
	throws RemoteException {
	try {
	    storedpart = new StorableObject(part);
	    this.crashcount = crashcount;
	} catch (RemoteException re) {
	    System.out.println("ParticipantHandle: ParticipantHandle: " +
				"Cannot store the TransactionParticipant");
	    re.printStackTrace();
	}
	this.prepstate = ACTIVE;
    }

    long getCrashCount() {
	return crashcount;
    }

    TransactionParticipant getParticipant() {
	TransactionParticipant tmp = null;
	try {
	    tmp = (TransactionParticipant) storedpart.get();
	} catch (Exception re) {
	    System.out.println("ParticipantHandle: getParticipant: " +
				"problem unpacking participant");
	    re.printStackTrace();
	}
	return tmp;
    }

    StorableObject getStoredPart() {
	return storedpart;
    }

    synchronized void setPrepState(int state) {
	switch (state) {
	    case PREPARED:
	    case NOTCHANGED:
	    case COMMITTED:
	    case ABORTED:
		break;
	    default:
		throw new IllegalArgumentException("ParticipantHandle: " +
			    "setPrepState: cannot set to " + 
		    	    com.sun.jini.constants.TxnConstants.getName(state));
	}

	this.prepstate = state;
    }

    synchronized int getPrepState() {
	return prepstate;
    }


    boolean compareTo(ParticipantHandle other) {
        if (storedpart != null)
	    return storedpart.equals(other);
	else if (other == null)
	    return false;

	return false;
    }
}
