/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahalo;

import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;

import com.sun.jini.mahalo.*;
import com.sun.jini.mahalo.log.*;
import com.sun.jini.thread.*;

import java.util.*;

import java.rmi.RemoteException;
import java.rmi.NoSuchObjectException;

/**
 * A <code>SettlerTask</code> is scheduled task, which
 * causes an unsettled transaction to settle.
 *
 * @author Gary Holness
 */
public class SettlerTask extends RetryTask implements TransactionConstants {
    private long tid;
    private int attempt;
    private int maxtries = Integer.MAX_VALUE;
    private TransactionManager txnmgr;

    private static final boolean DEBUG = false;

    /**
     * Constructs a <code>SettlerTask</code>.
     *
     * @param manager <code>TaskManager</code> providing the threads
     *                of execution.
     *
     * @param txnmgr <code>TransactionManager</code> which owns the
     *               the transaction.
     * 
     * @param tid transaction ID
     */
    public SettlerTask(TaskManager manager,
			    TransactionManager txnmgr, long tid) {
	super(manager);

	if (txnmgr == null)
	    throw new IllegalArgumentException("SettlerTask: SettlerTask: " +
					    "txnmgr must be non-null");
	this.txnmgr = txnmgr;
	this.tid = tid;
    }

    /**
     * Inherit doc comment from supertype.
     *
     * @see com.sun.jini.thread.RetryTask
     */
    public boolean runAfter(List list, int max) {
        return false;
    }

    public boolean tryOnce() {
        try {
	    if (attempt >= maxtries)
		return true;

	    attempt++;

	    if(DEBUG)
		System.out.println("SettlerTask: tryOnce: settling " +
				    "Transaction(" + tid + ")");

	    int state = txnmgr.getState(tid);
	    switch(state) {
	      case ACTIVE:
	      case COMMITTED:
                txnmgr.commit(tid, Long.MAX_VALUE);
		break;
	    
	      case ABORTED:
		txnmgr.abort(tid, Long.MAX_VALUE);
		break;
	    }

	} catch (NoSuchObjectException nsoe) {
        } catch (TransactionException te) {
	    if (DEBUG) {
                System.out.println("SettlerTask: tryOnce: " +
                                            te.getMessage());
                System.out.println("SettlerTask: " +
                   "tryOnce: cannot " +
                   "settle recovered transaction: " +
                                                tid);
                te.printStackTrace();
	    }
        } catch (RemoteException re) {
	    //try again
	    return false;
	}

            if(DEBUG)
                System.err.println("SettlerTask: tryOnce: SETTLED " +
                                    "Transaction(" + tid + ")");

	return true;
    }
}
