/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahout;

import com.sun.jini.mahout.PersistentServiceDesc;
import java.util.Enumeration;
import java.util.Properties;
import java.io.Serializable;

/**
 * Simple implementation of PersistentServiceDesc.  Its configuration
 * is fixed at creation.  Extends <code>PersistentServiceDesc</code>
 * with an additional method to return a lookup group list.
 */
public class BasicServiceDesc implements PersistentServiceDesc, Serializable
{
    final private String path;
    final private String classpath;
    final private String codebase;
    final private String policy;
    final private String jvmPath;
    final private String[] jvmArgs;
    final private String[] propvals;
    final private String[] propkeys;
    final private String[] groups;

    /**
     * Create a BasicServiceDesc that will return the given values
     * for getClassPath(), getCodebase(), getPolicyPath(), 
     * getPersistentsPath(), getGroups(), getJVMPath(), getJVMOptions(),
     * getProperties()
     */
    public BasicServiceDesc(String classpath, String codebase, String policy,
        String path, String[] groups, String jvmPath, String[] jvmArgs,
	Properties props)
    {
	this.classpath = classpath;
	this.codebase = codebase;
	this.policy = policy;
	this.path = path;
	this.groups = groups;
	this.jvmPath = jvmPath;
	this.jvmArgs = jvmArgs;


	String key = null;
	String value = null;

	if (props != null) {
	    propkeys = new String[props.size()];
	    propvals = new String[props.size()];

	    int i = 0;
	    for (Enumeration e = props.propertyNames();
			        e.hasMoreElements(); i++) {

	        key = (String) e.nextElement();
	        value = (String) props.getProperty(key);
	        propkeys[i] = key;
	        propvals[i] = value;
	    }
	} else {
	    //Satisfy the compiler
	    propkeys = null;
	    propvals = null;
	}
    }

    // purposefully inherit doc comment from supertype
    public String getPersistentsPath() {return path;}

    // purposefully inherit doc comment from supertype
    public String getClassPath() {return classpath;}

    // purposefully inherit doc comment from supertype
    public String getCodebase() {return codebase;}

    // purposefully inherit doc comment from supertype
    public String getPolicyPath() {return policy;}

    // purposefully inherit doc comment from supertype
    public Properties getProperties() {
	Properties props = null;
	
	if (propkeys != null ) {
	    props = new Properties();

	    for (int i = 0; i < propkeys.length; i++) {
	        props.setProperty(propkeys[i], propvals[i]);
	    }
	}

	return props;
    }

    // purposefully inherit doc comment from supertype
    public String getJVMPath() {return jvmPath;}

    // purposefully inherit doc comment from supertype
    public String[] getJVMOptions() {return jvmArgs;}
    
    /**
     * Lookup groups list, set when object was created.
     */
    public String[] getGroups() {return groups;}
}
