/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahout;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.AccessException;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.Remote;

/**
 * A <code>RegistryLocator</code> implements the
 * methods needed in order to locate a registry.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see Locator
 */

public class RegistryLocator extends Locator {
    private static boolean DEBUG = false;
    public RegistryLocator() {
    }

   /**
    *  This method returns null if nothing can
    *  be found.  The semantics are currently
    *  to return the registry running on the
    *  default port.
    */

    public Object locate() {
	Registry tmpobj = null;

	String locatehost = System.getProperty("outrigger.host");
	try {
	    Registry reg = null;
	    
	    if (DEBUG)
		System.out.println("RegistryLocator: locate: host = " + locatehost);

	    String portString = 
		System.getProperty("com.sun.jini.rmiRegistryPort");
	    int port;

	    reg = LocateRegistry.getRegistry(locatehost, calcPort());

            //Create and install the security manager
            try {
                System.setSecurityManager(new RMISecurityManager());
            } catch (SecurityException se) {
                 //swallow exception if security manager
                 //has already been installed
            }

	    tmpobj = reg;

	} catch (Exception e) {
	    System.out.println("RegistryLocator: find: " + e.getMessage());
	    e.printStackTrace();
	}

	if (DEBUG)
	    System.out.println("RegistryLocator: locate: registry = " + tmpobj);

	return tmpobj;
    }

    /**
     * Determine what port to look for the RMI registry on.
     */
    public static int calcPort() {
	String portString = 
	    System.getProperty("com.sun.jini.rmiRegistryPort");

	if (portString != null) {
	    Integer portObj = Integer.valueOf(portString);
	    return(portObj.intValue());
	} else
	    return(Registry.REGISTRY_PORT);
    }	
}
