/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mahout.binder;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.activation.ActivationException;
import java.rmi.activation.ActivationGroupID;
import net.jini.core.entry.Entry;

/**
 * BinderSetup implementation for lookup binders
 * @see BinderSetup
 * @author jmcclain
 */
public class LookupBinderSetup implements BinderSetup {
    /** This initial attribute list for the service */
    private Entry[] attributes;

    /** The Activation state for <code>RemoteJoinAdminImpl</code> */
    private JoinAdminActivationState state;

    /** 
     * Remote ref to <code>RemoteJoinAdminImpl</code> to be used as 
     * key to find actual local version when activated.
     */
    private LocalActivatableObjectTable.Findable key;

    /**
     * Create a <code>LookupBinderSetup</code>
     *
     * @param filePrefix String to prepend to any files/directories
     *                   placed in <code>storageDirectory</code>.
     * @param storageDirectory Path to directory where binder should
     *                         persist its data
     * @param codeSource Codesource where the underlying
     *                   <code>RemoteJoinAdminImpl</code> can find its classes.
     * @param gID Activation group where the
     *            <code>RemoteJoinAdminImpl</code> should be placed.
     * @param attributes Initial set of lookup attributes for the service.
     */
    public LookupBinderSetup(String filePrefix, String storageDirectory,
        String codeSource, ActivationGroupID gID, Entry[] attributes) 
	throws RemoteException, ActivationException, IOException
    {
	this.attributes = attributes;
	state = new JoinAdminActivationState(storageDirectory, filePrefix);
	key = RemoteJoinAdminImpl.createActivatable(state, codeSource, gID);
    }

    // Inherit doc comment from super interface
    public BinderFactory getBinderFactory(){
	return new BinderPackage(key);
    }

    // Inherit doc comment from super interface
    public void setService(Object service) 
	throws IOException, RemoteException 
    {
	state.populate(service, attributes);
	key.activate();
    }

    /**
     * Our implementation of <code>BinderFactory</code> to return from
     * <code>getBinderFactory()</code>
     * @see BinderSetup#getBinderFactory
     */
    private static class BinderPackage implements BinderFactory
    {
	static final long serialVersionUID = 3481267560121338832L;

	/** 
	 * Key to find <code>RemoteJoinAdminImpl</code> 
	 * @serial
	 */
	LocalActivatableObjectTable.Findable key;

	/** 
	 * Value of the last <code>get()</code> call 
	 * @serial
	 */
	Binder binder = null;

	/** 
	 * Create a binder package that will return a new
	 * <code>LookupBinder()</code> backed by the passed
	 * <code>RemoteJoinAdminImpl</code> when <code>get()</code> is
	 * called.
	 */
	BinderPackage(LocalActivatableObjectTable.Findable key) {
	    this.key = key;
	}

	// Inherit doc comment from super interface
	public Binder get() throws RemoteException
	{
	    if (binder != null)
		return binder;

	    binder = new LookupBinder(key);
	    return binder;
	}
    }
}
