/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mercury;

import com.sun.jini.proxy.UUID;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.List;
 
public class EventLogFactory {

    private HashMap iterators = new HashMap();

    /** Method invoked to recover persistent state upon startup */
    public void recover() throws IOException {};

    /** 
     * Method to return the iterator object for the designated ID. 
     * An iterator will be created if one doesn't already exist. 
     * Otherwise, the existing iterator associated with the provided 
     * <code>UUID</code> will be returned.
     */
    public EventLogIterator iterator(UUID uuid) {

        EventLogIterator eli = (EventLogIterator)iterators.get(uuid);
        if (eli == null) {
            eli = new EventLogIteratorImpl(uuid);
            iterators.put(uuid, eli);
	}
            
	return eli;
    }

    private static class EventLogIteratorImpl 
	implements EventLogIterator, Serializable 
    {
      
        private final UUID uuid;
        private transient List entries = null; 

        /** Simple constructor */
        EventLogIteratorImpl(UUID id) {
            uuid = id;
	    entries = Collections.synchronizedList(new LinkedList());
        }

        // Inherit documentation from supertype
        public void add(Object o) throws IOException, IllegalArgumentException {
            entries.add(o);
	}
            
        // Inherit documentation from supertype
        public boolean hasNext() {
            return !entries.isEmpty();
        }

        // Inherit documentation from supertype
        public Object next() throws IOException, NoSuchElementException {
            return entries.get(0);
        }

        // Inherit documentation from supertype
        public void remove() throws IOException, IllegalStateException {
            entries.remove(0);
        }

        // Inherit documentation from supertype
        public void destroy() throws IOException {
            entries.clear();
        }

        public int size() {
            return entries.size();
        }
        
        private void writeObject(ObjectOutputStream os) throws IOException {
            os.defaultWriteObject();
        }

        private void readObject(ObjectInputStream is) 
            throws IOException, ClassNotFoundException {
            is.defaultReadObject();
	    entries = Collections.synchronizedList(new LinkedList());
        }
    }
}


