/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mercury;

import java.io.IOException;
import java.util.NoSuchElementException;

/** 
 * EventLogIterator provides an abstraction for accessing the events
 * stored on behalf of a particular registration.  The methods closely
 * resemble the <code>java.util.iterator</code> interface with the 
 * exception of the the throws clauses and the additional destroy
 * method.
 */
public interface EventLogIterator {
    /** Add the given object to the collection of stored events */
    void add(Object o) throws IOException, IllegalArgumentException;

    /**  
     * Return true if there are any events in the collection and 
     * false otherwise.
     */
    boolean hasNext();

    /** 
     * Return the next event in the collection. It is expected that
     * that the iterator's cursor is not modified at this point.
     */
    Object next() throws IOException, NoSuchElementException;

    /** 
     * Remove the event at the iterator's current cursor position.
     * It is expected that the cursor position will be updated to 
     * point to the next event object, if any.
     */
    void remove() throws IOException, IllegalStateException;

    /** Destroy the collection of stored events. */
    void destroy() throws IOException;

    // TODO - remove this method in final release
    int size();
}

