/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mercury;

import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.proxy.ThrowThis;
import com.sun.jini.proxy.UUID;

import java.rmi.Remote;
import java.rmi.RemoteException;

import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.UnknownEventException;
import net.jini.event.MailboxRegistration;
import net.jini.admin.Administrable;

/**
 * MailboxBackEnd defines the private protocol between the various client-side
 * proxies and the event mailbox server.
 * <p>
 * The declared methods are pretty straightforward mappings of the
 * EventMailbox and MailboxRegistration interfaces.
 * <p>
 * Note: The Landlord interface extends Remote, which implicitly makes this 
 * interface Remote as well. 
 */
interface MailboxBackEnd extends Landlord, Administrable, MailboxAdmin {

    /** Property used for generating debugging information */
    String DEBUG_PROP = "com.sun.jini.mercury.debug";

    /**
     * Returns the unique identifier generated (or recovered) by the back-end
     * implementation of the event mailbox service when an instance of
     * that service is constructed. This ID is typically used to determine
     * equality between the proxies of any two instances of the event
     * mailbox service.
     * 
     * @return the unique ID that was generated (or recovered) by the
     *         back-end implementation of the event mailbox service
     *         at creation time
     * 
     * @throws java.rmi.RemoteException typically, this exception occurs when
     *         there is a communication failure between the client and the
     *         event mailbox service.
     */
    public UUID getProxyID() throws RemoteException;

    /**
     * Register with the event mailbox service for the requested duration.
     * @see net.jini.event.EventMailbox#register
     */
    MailboxRegistration register(long leaseDuration)
	throws RemoteException;

    /**
     * Enable deliver of events for the given registration
     * to the specified target
     *
     * @param uuid The unique registration identifier
     *
     * @param target The designated delivery target for event notifications
     */
    public void enableDelivery(UUID registrationID, RemoteEventListener target) 
	throws RemoteException, ThrowThis;
	
    /**
     * Disable deliver of events for the given registration
     *
     * @param uuid The unique registration identifier
     */
    public void disableDelivery(UUID registrationID) 
	throws RemoteException, ThrowThis;

    /**
     * Collect remote event on behalf of the associated registration.
     *
     * @param uuid The unique registration identifier
     *
     * @param theEvent The event to store and/or forward
     */
    public void notify(UUID registrationID, RemoteEvent theEvent) 
	throws UnknownEventException, RemoteException, ThrowThis;
}
