/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mercury;

import com.sun.jini.proxy.UUID;

import java.io.Serializable;
import java.rmi.RemoteException;

import net.jini.admin.Administrable;
import net.jini.event.EventMailbox;
import net.jini.event.MailboxRegistration;

/**
 * A MailboxProxy is a proxy for the event mailbox service.  
 * This is the object passed to client of this service.
 * It implements the <code>EventMailbox</code> and the 
 * <code>Administrable</code>
 * interfaces.
 */
class MailboxProxy implements EventMailbox, Administrable, Serializable {

    /**
     * The reference to the event mailbox service implementation
     *
     * @serial
     */
    private final MailboxBackEnd server;

    /**
     * The proxy's <code>UUID</code>
     *
     * @serial
     */
    private final UUID proxyID;

    /** Simple constructor. */
    public MailboxProxy(MailboxBackEnd server, UUID proxyID) {
	this.server = server;
	this.proxyID = proxyID;
    }

    // inherit javadoc from parent
    public MailboxRegistration register(long duration) throws RemoteException {
        // Check for a bad argument
        // Note that -1 (i.e. Lease.ANY) is a valid request
        if (duration < -1)
            throw new IllegalArgumentException(
                "Duration values must be positive");
        return server.register(duration);
    }

    // inherit javadoc from parent
    public Object getAdmin() throws RemoteException {
        return server.getAdmin();
    }

    /** Proxies for servers with the same proxyID have the same hash code. */
    public int hashCode() {
	return proxyID.hashCode();
    }

    /** Proxies for servers with the same proxyID are considered equal. */
    public boolean equals(Object obj) {
        // TODO - check for null
	return (obj instanceof MailboxProxy &&
		proxyID.equals(((MailboxProxy)obj).proxyID));
    }

}
