/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.mercury;

import java.io.Serializable;
import java.rmi.RemoteException;

import com.sun.jini.proxy.ThrowThis;
import com.sun.jini.proxy.UUID;

import net.jini.event.MailboxRegistration;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.lease.Lease;


class Registration implements MailboxRegistration, Serializable {

    /** Unique identifier for this registration */
    private final UUID registrationID;

    /** Reference to service implementation */
    private final MailboxBackEnd mailbox;

    /** Reference to RemoteEventListener implementation */
    private final ListenerProxy listener;

    /** The service's registration lease */ 
    private final Lease lease;

    /** Simple constructor */
    public Registration(UUID id, MailboxBackEnd srv, Lease l) {
        if (id == null || srv == null || l == null)
            throw new IllegalArgumentException("Cannot accept null arguments");
        registrationID = id;
        mailbox = srv;
        listener = new ListenerProxy(srv, id);
        lease = l;
    }

    // inherit javadoc from supertype
    public Lease getLease() {
	return lease;
    }
    
    // inherit javadoc from supertype
    public RemoteEventListener getListener() {
	return (RemoteEventListener)listener;
    }

    // inherit javadoc from supertype
    public void enableDelivery(RemoteEventListener target) 
	throws RemoteException
    {
	if (target == null) { // same as disableDelivery
	    try {
                mailbox.disableDelivery(registrationID);
	    } catch (ThrowThis tt) {
	        tt.throwRemoteException();
	    }
	} else if ((target instanceof ListenerProxy) &&
	           (listener.equals((ListenerProxy)target))) {
	    // Can't re-submit a mailbox provided listener
	    throw new IllegalArgumentException("Cannot provide an " +
		"a target that was provided by the EventMailbox service");
	} else { // OK to make the call, now
	    try {
	        mailbox.enableDelivery(registrationID, target);
	    } catch (ThrowThis tt) { 
	        tt.throwRemoteException();
	    }
	}
    }

    // inherit javadoc from supertype
    public void disableDelivery() throws RemoteException {
	try { 
            mailbox.disableDelivery(registrationID);
	} catch (ThrowThis tt) { 
	    tt.throwRemoteException();
	}
    }

    /** Proxies with the same registrationID have the same hash code. */
    public int hashCode() {
        return registrationID.hashCode();
    }

    /** Proxies with the same registrationID are considered equal. */
    public boolean equals(Object obj) {
        return (obj instanceof Registration &&
                registrationID.equals(((Registration)obj).registrationID));
    }


}
