/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.norm;

import net.jini.core.lease.Lease;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;

/**
 * Object that transfers events from the Lease Renewal Manager to the
 * rest of Norm Server.  Isolates the renewal manager from having to
 * block on the snap shot locks.
 */
class FailureNotifer extends Thread implements LeaseListener {
    /** Ref to the main server object which has all the top level methods */
    final private NormServerImpl server;

    /**
     * Queue we use to decouple the reception of events from the lease
     * renewal manager from the scheduling of the sending of remote
     * events and modifying our internal table s (which both require
     * obtaining serious locks.
     */
    final private Queue queue = new Queue();

    /**
     * Simple constructor
     * @param server Object that will make the will make the actual
     * internal updates and schedule the sending of remote events.
     */
    FailureNotifer(NormServerImpl server) {
	super("Failure Notifier");
	setDaemon(true);
	this.server = server;	
    }

    //////////////////////////////////////////////////
    // Methods required by the LeaseListener interface 

    // Inherit java doc from super type
    public void notify(LeaseRenewalEvent e) {	
	// Paranoia, check to make sure that lease is one of wrapped
	// client lease...if its not ignore the event
	final Lease l = e.getLease();
	if (l instanceof ClientLeaseWrapper) {
	    ClientLeaseWrapper clw = (ClientLeaseWrapper)l;
	    queue.enque(clw);
	}
    }

    public void run() {
	// Loop taking items off the queue and pass them to the server
	while (true) 
	    try {
		final ClientLeaseWrapper clw 
		    = (ClientLeaseWrapper) queue.deque();
		server.renewalFailure(clw);
	    } catch (InterruptedException e) {
		// Some wants this tread dead, just return
		return;
	    } catch (RuntimeException e) {
		System.err.println("Failure Notifier received the following " +
				   "exception:");
		e.printStackTrace();
		System.err.println("When processing a renewal failure, " +
				   "trying to go on");
	    } catch (Error e) {
		System.err.println("Failure Notifier received the following " +
				   "exception:");
		e.printStackTrace();
		System.err.println("When processing a renewal failure, " +
				   "trying to go on");
	    }
    }
}
