/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.norm;

import java.io.IOException;
import com.sun.jini.mahout.ExecUtilBase;

/**
 * Utility class for starting Norm Lease Renewal Servers
 */
public class NormExecUtil extends ExecUtilBase {
    /** 
     * Change codebase, classpath, and/or policyFilePath the
     * appropriate <code>com.sun.jini.exec.norm.*</code> property has
     * been set
     */
    private void constructorUtil() {
	final String normCodebase = 
	    System.getProperty("com.sun.jini.exec.norm.codebase");

	if (codebase == null && normCodebase == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a codebase using com.sun.jini.exec.norm.codebase or " +
		 "com.sun.jini.exec.codebase");
	
	if (normCodebase != null) codebase = normCodebase;


	final String normClasspath = 
	    System.getProperty("com.sun.jini.exec.norm.class.path");

	if (classpath == null && normClasspath == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a classpath using com.sun.jini.exec.norm.class.path or " +
		 "com.sun.jini.exec.class.path");
	
	if (normClasspath != null) classpath = normClasspath;



	final String normPolicy = 
	    System.getProperty("com.sun.jini.exec.norm.policy");

	if (policyFilePath == null && normPolicy == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a policy file using com.sun.jini.exec.norm.policy or " +
		 "com.sun.jini.exec.policy");
	
	if (normPolicy != null) policyFilePath = normPolicy;
    }

    /**
     * Set <code>cmd</code>
     * @param groups      Lookup groups the service should bind into. Ignored
     *                    if <code>useRegistry</code> is <code>true</code>.
     */
    private void buildCmd(String groups)
    {
	cmd += "-Djava.security.policy=" + policyFilePath  + " ";
	cmd += "-Djava.class.path=" + classpath + " ";
	cmd += "com.sun.jini.norm.NormServerImpl " + codebase + " " + 
	    policyFilePath + " " + persistencePath + " " + groups;
    }

    /**
     * Build an MahaloExecUtil for a Transaction Manager that will bind itself
     * into a lookup service
     *
     * @param groups Lookup group list
     * @param name   The transaction manager's name
     */
    public NormExecUtil(String groups) throws IOException 
    {
	super("Norm");
	constructorUtil();
	buildCmd(groups);
    }
}
