/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.norm;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;

import net.jini.admin.Administrable;
import net.jini.core.lease.Lease;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.RemoteEventListener;
import net.jini.lease.LeaseRenewalService;

import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.proxy.UUID;
import com.sun.jini.proxy.ThrowThis;

import com.sun.jini.mahout.KillVMAdmin;

/**
 * This interface is the private wire protocol to that the various
 * proxy objects a Norm server creates (the lease renewal service
 * itself, sets, leases, admins, etc.) use to communicate back to the
 * server.
 */
public interface NormServer extends Landlord, LeaseRenewalService, 
    Administrable, NormAdmin, KillVMAdmin, Remote 
{
    /**
     * If calling <code>setExpirationWarningListener</code> with a
     * <code>null</code> listener this is the value that should be passed 
     */
    final static long NO_LISTENER = -1;

    /**
     * Add a lease to a set.
     * @param id What set the lease should be added to.
     * @param leaseToRenew the lease to be added to the set
     * @param membershipDuration how long the lease should be in the set
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException */
    public void renewFor(UUID id, Lease leaseToRenew,
			 long membershipDuration)
	throws RemoteException, ThrowThis;

    /**
     * Remove a lease from a set
     * @param id of set is being operated on.
     * @param leaseToRemove the lease to be added to the set
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException 
     */
    public Lease remove(UUID id, Lease leaseToRemove) 
	throws RemoteException, ThrowThis;

    /**
     * Set the expiration warning listener for a set.  Also used to 
     * cancel a registration.
     * @param id of set is being operated on.
     * @param listener Listener to be notified when this event occurs.
     *                 Pass <code>null</code> to clear the registration.
     * @param minWarning How long be for the lease on the set expires
     *                   should the event be sent. Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @param handback   An object to be handed back to the listener
     *                   when the warning event occurs. Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @return An <code>EventRegistration</code> object for the new
     * registration if <code>listener</code> is non-<code>null</code>
     * and <code>null</code> otherwise.
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException
     */
    public EventRegistration setExpirationWarningListener(
			         UUID                id,
	                         RemoteEventListener listener, 
				 long                minWarning, 
				 MarshalledObject    handback)
	throws RemoteException, ThrowThis;

    /**
     * Set the renewal failure listener for a set.  Also used to 
     * cancel a registration.
     * @param id of set is being operated on.
     * @param listener Listener to be notified when this event occurs.
     *                 Pass <code>null</code> to clear the registration.
     * @param handback   An object to be handed back to the listener
     *                   when the failure event occurs.  Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @return An <code>EventRegistration</code> object for the new
     * registration if <code>listener</code> is non-<code>null</code>
     * and <code>null</code> otherwise.
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException      
     */
    public EventRegistration setRenewalFailureListener(
			         UUID                id,
	                         RemoteEventListener listener, 
				 MarshalledObject    handback)
	throws RemoteException, ThrowThis;


}


