/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.norm.event;

import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;

/**
 * Object associated with an <code>EventType</code> so it can ensure
 * that the lease on an event notification is still valid and notify
 * the client when a exception occurs in the course of attempting to
 * send an event.
 *
 * @see EventType
 * @see EventTypeFactory 
 */
public interface SendMonitor {
    /**
     * Method called when an attempt to send the event associated with
     * this object results in a definite exception
     * (e.g. <code>java.rmi.NoSuchObjectException</code> or any other
     * <code>Throwable</code> that is not a subclass of
     * <code>java.rmi.RemoteException</code>.
     * <p>
     * The caller will own no locks when calling this method.
     * @param type The object that generated the event
     * @param listener The listener the event is intended for.
     * @param ev   The remote event that could not be sent.
     * @param t    The definite exception that caused us to give up
     *             sending the event.  */
    public void definiteException(EventType           type,
                                  RemoteEventListener listener,
                                  RemoteEvent         ev,
                                  Throwable           t);
    /**
     * Should return <code>true</code> if the lease associated with this
     * event is still valid and <code>false</code> otherwise.
     * <p>
     * The caller will own no locks when calling this method.
     */
    public boolean isCurrent();
}
