/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */
package com.sun.jini.reggie;

import net.jini.core.lease.*;

/** The base class for lease proxies. */
abstract class RegistrarLease extends com.sun.jini.lease.AbstractLease {

    private static final long serialVersionUID = 1286538697644640310L;

    /**
     * The registrar.
     *
     * @serial
     */
    protected final Registrar server;
    /**
     * The internal lease id.
     *
     * @serial
     */
    protected final long leaseID;

    /** Simple constructor. */
    protected RegistrarLease(Registrar server, long leaseID, long expiration) {
	super(expiration);
	this.server = server;
	this.leaseID = leaseID;
    }

    /** Create a lease map. */
    public LeaseMap createLeaseMap(long duration) {
	return new RegistrarLeaseMap(this, duration);
    }

    /**
     * Two leases can be batched if they are both RegistrarLeases and
     * have the same server.
     */
    public boolean canBatch(Lease lease) {
	return (lease instanceof RegistrarLease &&
		server.equals(((RegistrarLease)lease).server));
    }

    /** Return the registrar. */
    Registrar getRegistrar() {
	return server;
    }

    /** Returns the lease ID */
    long getLeaseID() {
	return leaseID;
    }

    /** Returns the service ID, or the event ID as a Long */
    abstract Object getRegID();

    /** Set the expiration. */
    void setExpiration(long expiration) {
	this.expiration = expiration;
    }
}
