/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JSK
 */

package com.sun.jini.reliableLog;

import java.io.*;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * A LogHandler represents snapshots and update records as serializable
 * objects.
 * <p>
 * This implementation does not know how to create an initial snaphot or
 * apply an update to a snapshot.  The client must specify these methods
 * via a subclass.
 *
 * @see ReliableLog
 */
public abstract class LogHandler {

    /**
     * Creates a LogHandler for a ReliableLog.
     */
    public LogHandler() {}
    
    /**
     * Writes the snapshot to a stream.  This callback is invoked when
     * the client calls the snaphot method of ReliableLog.
     *
     * @param out the output stream
     *
     * @exception Exception can raise any exception
     */
    public abstract void snapshot(OutputStream out) throws Exception;
    
    /**
     * Read the snapshot from a stream.  This callback is invoked when
     * the client calls the recover method of ReliableLog.  
     *
     * @param in the input stream
     *
     * @exception Exception can raise any exception
     */
    
    public abstract void recover(InputStream in) throws Exception;
    
    /**
     * Writes the representation (a serializable object) of an update 
     * to a stream.  This callback is invoked when the client calls the 
     * update method of ReliableLog.
     *
     * @param out the output stream
     * @param value the update object
     *
     * @exception Exception can raise any exception
     */
    public void writeUpdate(OutputStream out, Object value) throws Exception {
	ObjectOutputStream s = new ObjectOutputStream(out);
 	s.writeObject(value);
	s.flush();
    }
    
    /**
     * Reads a stably logged update (a serializable object) from a
     * stream.  This callback is invoked during recovery, once for
     * every record in the log.  After reading the update, this method
     * invokes the applyUpdate (abstract) method in order to execute
     * the update.
     *
     * @param in the input stream
     *
     * @exception Exception can raise any exception
     */
    public void readUpdate(InputStream in) throws Exception {
	ObjectInputStream  s = new ObjectInputStream(in);
	applyUpdate(s.readObject());
    }

    /**
     * Reads a stably logged update (a serializable object) from a stream.  
     * This callback is invoked during recovery, once for every record in the 
     * log.  After reading the update, this method is invoked in order to
     * execute the update.
     *
     * @param update the update object
     *
     * @exception Exception can raise any exception
     */
    public abstract void applyUpdate(Object update) throws Exception;
    
}
