/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JXP
 */

package net.jini.lease;

import net.jini.core.lease.Lease;

/** 
 * With respect to an entity that uses the <code>LeaseRenewalManager</code>
 * to manage leases granted to the entity, this interface defines the
 * mechanism through which the entity receives notification that the 
 * <code>LeaseRenewalManager</code> has failed to renew one of the leases.
 * Such renewal failures typically occur because of one of the following
 * conditions:
 * <ul>
 * <li> the expiration time of the lease has been reached, and the lease
 *      has already expired, before the renewal request can be made
 * <li> the renewal request has been denied by the lease grantor for
 *      some reason, even though the desired absolute expiration time
 *      of the lease has not yet been reached
 * <li> the renewal request could not be completed before the expiration
 *      of the lease
 * </ul>
 * It is the responsibility of the entity to register with the 
 * <code>LeaseRenewalManager</code>, a reference to an object that
 * implements this interface. That object should define the actions to
 * take upon receipt of such notifications. Then, when one of the above 
 * conditions occurs, the <code>LeaseRenewalManager</code> will send an 
 * instance of the <code>LeaseRenewalEvent</code> class to that listener
 * object. Note that prior to sending the event, the 
 * <code>LeaseRenewalManager</code> will remove the affected lease from
 * its managed set of leases.
 * 
 * @see net.jini.core.Lease
 * @see net.jini.lease.LeaseRenewalManager
 * @see net.jini.lease.LeaseRenewalEvent
 */
public interface LeaseListener extends java.util.EventListener {
    /**
     * Called by the <code>LeaseRenewalManager</code> when it fails to
     * renew a lease because: the expiration time of the lease has been
     * reached (and the lease has already expired) before the renewal
     * request can be made; the renewal request has been denied by the
     * lease grantor, even even though the desired absolute expiration time
     * of the lease has not yet been reached; or the renewal request could
     * not be completed before the expiration of the lease.
     * <p>
     * Note that prior to invoking this method, the
     * <code>LeaseRenewalManager</code> removes the affected lease
     * from the managed set of leases. Note also that because of the
     * re-entrancy guarantee made by the <code>LeaseRenewalManager</code>,
     * new leases can be safely added from the within any listeners that
     * implement this interface.
     * <p>
     * @param e  instance of <code>LeaseRenewalEvent</code> containing
     *           information about the lease on which a renewal failure
     *           occurred, as well as information about the condition that 
     *           caused the renewal attempt to fail
     */
    void notify(LeaseRenewalEvent e);
}
