/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JXP
 */
package net.jini.lease;

import java.rmi.RemoteException;

/**
 * Defines the interface to the lease renewal service.  The interface
 * is not a remote interface; each implementation of the renewal
 * service exports proxy objects that implement the
 * LeaseRenewalService interface that use an implementation-specific
 * protocol to communicate with the actual remote server.  All of the
 * proxy methods obey normal RMI remote interface semantics.  Two
 * proxy objects are equal if they are proxies for the same renewal
 * service.  Every method invocation (on both LeaseRenewalService and
 * LeaseRenewalSet) is atomic with respect to other invocations.
 * 
 * @see LeaseRenewalSet 
 */

public interface LeaseRenewalService {
    /**
     * Create a new LeaseRenewalSet that the client can place leases
     * to be renewed in.
     *
     * @param leaseDuration requested lease duration in milliseconds.
     * @return A new LeaseRenewalSet in the renewal service
     *
     * @throws IllegalArgumentException if <code>leaseDuration</code>
     * is not positive, <code>Lease.ANY</code>, or
     * <code>Lease.FOREVER</code>.
     *
     * @throws java.rmi.RemoteException
     */
    public LeaseRenewalSet createLeaseRenewalSet(long leaseDuration) 
	throws RemoteException;
}

