/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * CopyrightVersion 1.1Alpha_Jini
 */
package com.sun.jini.tool;

import sun.tools.java.Environment;
import sun.tools.java.Identifier;
import sun.tools.java.BinaryClass;
import sun.tools.java.ClassDeclaration;
import sun.tools.java.ClassNotFound;
import sun.tools.java.Package;
import sun.tools.java.Type;
import sun.tools.java.ClassPath;
import sun.tools.java.ClassFile;
import sun.tools.java.MemberDefinition;
import sun.tools.java.Constants;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.text.MessageFormat;
import java.io.File;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.BufferedInputStream;

public class ClassDep {
    private final HashSet seen = new HashSet();
    private Env env;
    public String classpath = "";
    public boolean edges = false;
    public boolean ignoreOuter = true;
    public final ArrayList inside = new ArrayList();
    public final ArrayList classes = new ArrayList();
    public final ArrayList roots = new ArrayList();
    public final ArrayList prunes = new ArrayList();
    public final ArrayList skips = new ArrayList();
    public final ArrayList tells = new ArrayList();
    private final ArrayList results = new ArrayList();

    private boolean isInside(String n) {
	for (int i = 0; i < inside.size(); i++) {
	    if (n.startsWith((String)inside.get(i)))
		return true;
	}
	return false;
    }

    private void traverse(String path) {
	for (int i = 0; i < prunes.size(); i++) {
	    if (path.endsWith((String)prunes.get(i)))
		return;
	}
	String[] files = new File(path).list();
	if (files == null)
	    return;
    outer:
	for (int i = 0; i < files.length; i++) {
	    String file = files[i];
	    if (!file.endsWith(".class")) {
		traverse(path + File.separatorChar + file);
	    } else {
		file = path + File.separatorChar +
		       file.substring(0, file.length() - 6);
		for (int j = 0; j < skips.size(); j++) {
		    String skip = (String)skips.get(j);
		    int k = file.indexOf(skip);
		    if (k < 0)
			continue;
		    k += skip.length();
		    if (file.length() == k || file.charAt(k) == '$')
			continue outer;
		}
		for (int j = 0; j < inside.size(); j++) {
		    int k = file.indexOf(File.separatorChar +
					 ((String)inside.get(j)).replace(
						     '.', File.separatorChar));
		    if (k >= 0)
			classes.add(file.substring(k + 1).replace(
						   File.separatorChar, '.'));
		}
	    }
	}
    }

    private void process(Identifier from, Type type) {
	while (type.isType(Constants.TC_ARRAY))
	    type = type.getElementType();
	if (type.isType(Constants.TC_CLASS))
	    process(from, type.getClassName(), false);
    }

    private void process(Identifier from, Identifier id, boolean top) {
	if (from != null) {
	    for (int i = 0; i < tells.size(); i++) {
		if (id.toString().equals((String)tells.get(i))) {
		    if (tells.size() > 1)
			print("classdep.cause", id, from);
		    else
			print("classdep.cause1", from);
		}
	    }
	}
	String n = id.toString();
	if (n.charAt(0) == '[') {
	    int i = 1;
	    while (n.charAt(i) == '[')
		i++;
	    if (n.charAt(i) == 'L')
		process(from,
			Identifier.lookup(n.substring(i + 1, n.length() - 1)),
			false);
	    return;
	}
	if (seen.contains(id))
	    return;
	boolean in = isInside(n);
	if (!in && !edges)
	    return;
	seen.add(id);
	if (in != edges)
	    results.add(Type.mangleInnerType(id).toString());
	if (!in && edges)
	    return;
	id = env.resolvePackageQualifiedName(id);
	BinaryClass cdef;
	try {
	    cdef = (BinaryClass)env.getClassDefinition(id);
	    cdef.loadNested(env);
	} catch (ClassNotFound e) {
	    print("classdep.notfound", id);
	    return;
	} catch (IllegalArgumentException e) {
	    print("classdep.illegal", id, e.getMessage());
	    return;
	} catch (Exception e) {
	    print("classdep.failed", id);
	    e.printStackTrace();
	    return;
	}
	cdef.loadNested(env);
	Identifier outer = null;
	if (ignoreOuter && cdef.isInnerClass() && cdef.isStatic())
	    outer = cdef.getOuterClass().getName();
	for (Enumeration enum = cdef.getDependencies();
	     enum.hasMoreElements(); )
	{
	    Identifier dep = ((ClassDeclaration)enum.nextElement()).getName();
	    if (outer != dep)
		process(id, dep, false);
	}
	for (MemberDefinition mem = cdef.getFirstMember();
	     mem != null;
	     mem = mem.getNextMember())
	{
	    if (mem.isVariable()) {
		process(id, mem.getType());
	    } else if (mem.isMethod() || mem.isConstructor()) {
		Type[] args = mem.getType().getArgumentTypes();
		for (int i = 0; i < args.length; i++) {
		    process(id, args[i]);
		}
		process(id, mem.getType().getReturnType());
	    }
	}
    }

    public String[] compute() {
	env = new Env(classpath);
	for (int i = 0; i < roots.size(); i++) {
	    traverse((String)roots.get(i));
	}
	for (int i = 0; i < classes.size(); i++) {
	    process(null, Identifier.lookup((String)classes.get(i)), true);
	}
	if (!tells.isEmpty())
	    return new String[0];
	String[] vals = (String[])results.toArray(new String[results.size()]);
	Arrays.sort(vals);
	return vals;
    }

    public static void usage() {
	print("classdep.usage", null);
    }

    private static ResourceBundle resources;
    private static boolean resinit = false;

    private static String getString(String key) {
	if (!resinit) {
	    try {
		resources = ResourceBundle.getBundle("com.sun.jini.tool.resources.classdep");
		resinit = true;
	    } catch (MissingResourceException e) {
		e.printStackTrace();
	    }
	}
	try {
	    return resources.getString(key);
	} catch (MissingResourceException e) {
	    return null;
	}
    }

    private static void print(String key, Object val) {
	String fmt = getString(key);
	if (fmt == null)
	    fmt = "no text found: \"" + key + "\" {0}";
	System.err.println(MessageFormat.format(fmt, new Object[]{val}));
    }

    private static void print(String key, Object val1, Object val2) {
	String fmt = getString(key);
	if (fmt == null)
	    fmt = "no text found: \"" + key + "\" {0} {1}";
	System.err.println(MessageFormat.format(fmt,
						new Object[]{val1, val2}));
    }

    public static void main(String[] args) {
	if (args.length == 0) {
	    usage();
	    return;
	}
	ClassDep dep = new ClassDep();
	boolean files = false;
	for (int i = 0; i < args.length ; i++ ) {
	    String arg = args[i];
	    if (arg.equals("-cp")) {
		i++;
		dep.classpath = args[i];
	    } else if (arg.equals("-files")) {
		files = true;
	    } else if (arg.equals("-in")) {
		i++;
		arg = args[i];
		if (!arg.endsWith("."))
		    arg = arg + '.';
		dep.inside.add(arg);
	    } else if (arg.equals("-edges")) {
		dep.edges = true;
	    } else if (arg.equals("-outer")) {
		dep.ignoreOuter = false;
	    } else if (arg.equals("-prune")) {
		i++;
		arg = args[i];
		if (arg.endsWith("."))
		    arg = arg.substring(0, arg.length() - 1);
		arg = File.separator + arg.replace('.', File.separatorChar);
		dep.prunes.add(arg);
	    } else if (arg.equals("-skip")) {
		i++;
		arg = args[i];
		if (arg.endsWith("."))
		    arg = arg.substring(0, arg.length() - 1);
		else
		    dep.seen.add(Identifier.lookup(arg));
		arg = File.separator + arg.replace('.', File.separatorChar);
		dep.skips.add(arg);
	    } else if (arg.equals("-tell")) {
		i++;
		dep.tells.add(args[i]);
	    } else if (arg.indexOf(File.separator) >= 0) {
		if (arg.endsWith(File.separator))
		    arg = arg.substring(0, arg.length() - 1);
		dep.roots.add(arg);
	    } else if (arg.startsWith("-")) {
		usage();
	    } else {
		dep.classes.add(arg);
	    }
	}
	String[] vals = dep.compute();
	for (int i = 0; i < vals.length; i++) {
	    if (files)
		System.out.println(vals[i].replace('.', File.separatorChar) +
				   ".class");
	    else
		System.out.println(vals[i]);
	}
    }

    private static class Env extends Environment {
	private final ClassPath noPath = new ClassPath("");
	private final ClassPath path;
	private final HashMap packages = new HashMap();
	private final HashMap classes = new HashMap();

	public Env(String classpath) {
	    path = new ClassPath(System.getProperty("sun.boot.class.path") +
				 File.pathSeparator +
				 System.getProperty("java.ext.dirs") +
				 File.pathSeparator +
				 classpath);
	}

	public int getFlags() {
	    return 0;
	}

	public boolean classExists(Identifier id) {
	    if (id.isInner())
		id = id.getTopName();
	    Type t = Type.tClass(id);
	    try {
		ClassDeclaration c = (ClassDeclaration)classes.get(t);
		if (c == null) {
		    Package pkg = getPackage(id.getQualifier());
		    return pkg.getBinaryFile(id.getName()) != null;
		}
		return c.getName().equals(id);
	    } catch (IOException e) {
		return false;
	    }
	}

	public ClassDeclaration getClassDeclaration(Identifier id) {
	    return getClassDeclaration(Type.tClass(id));
	}

	public ClassDeclaration getClassDeclaration(Type t) {
	    ClassDeclaration c = (ClassDeclaration)classes.get(t);
	    if (c == null) {
		c = new ClassDeclaration(t.getClassName());
		classes.put(t, c);
	    }
	    return c;
	}

	public Package getPackage(Identifier pkg) throws IOException {
	    Package p = (Package)packages.get(pkg);
	    if (p == null) {
		p = new Package(noPath, path, pkg);
		packages.put(pkg, p);
	    }
	    return p;
	}

	BinaryClass loadFile(ClassFile file) throws IOException {
	    DataInputStream in =
		new DataInputStream(new BufferedInputStream(
						     file.getInputStream()));
	    try {
		return BinaryClass.load(new Environment(this, file), in,
					ATT_ALLCLASSES);
	    } catch (ClassFormatError e) {
		throw new IllegalArgumentException("ClassFormatError: " +
						   file.getPath());
	    } catch (Exception e) {
		e.printStackTrace();
		return null;
	    } finally {
		in.close();
	    }
	}

	public void loadDefinition(ClassDeclaration c) {
	    Identifier id = c.getName();
	    if (c.getStatus() != CS_UNDEFINED)
		throw new IllegalArgumentException("No file for: " + id);
	    Package pkg;
	    try {
		pkg = getPackage(id.getQualifier());
	    } catch (IOException e) {
		throw new IllegalArgumentException("IOException: " +
						   e.getMessage());
	    }
	    ClassFile file = pkg.getBinaryFile(id.getName());
	    if (file == null)
		throw new IllegalArgumentException("No file for: " +
						   id.getName());
	    BinaryClass bc;
	    try {
		bc = loadFile(file);
	    } catch (IOException e) {
		throw new IllegalArgumentException("IOException: " +
						   e.getMessage());
	    }
	    if (bc == null)
		throw new IllegalArgumentException("No class in: " +
						   file);
	    if (!bc.getName().equals(id))
		throw new IllegalArgumentException("Wrong class in: " +
						   file);
	    c.setDefinition(bc, CS_BINARY);
	    bc.loadNested(this, ATT_ALLCLASSES);
	}
    }
}
