/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License"); 
 * you may not use this file except in compliance with the License. You 
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the  
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT 
 *                   Production Release (Final) on Microsoft Windows 
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.1Alpha_JXP
 */


package net.jini.lookup;

import  net.jini.core.lookup.ServiceItem;

/**
 * The <code>ServiceItemFilter</code> interface defines the methods
 * used by an object such as the <code>ClientLookupManager</code> or
 * the<code>LookupCache</code> to apply additional matching criteria 
 * when searching for services in which an entity has registered 
 * interest. It is the responsibility of the entity requesting the
 * application of additional criteria to construct an implementation of
 * this interface that defines the additional criteria, and to pass the
 * resulting object (referred to as a <i>filter</i>) into the object
 * that will apply it.
 * <p> 
 * The filtering mechanism provided by implementations of this
 * interface is particularly useful to entities that wish to extend the
 * capabilities of the standard template matching scheme. For example,
 * because template matching does not allow one to search for services
 * based on a range of attribute values, this additional matching
 * mechanism can be exploited by the entity to ask the managing object
 * to find all registered printer services that have a resolution
 * attribute between say, 300 dpi and 1200 dpi.
 *
 * @see ClientLookupManager
 * @author Zane Pan
 */
public interface ServiceItemFilter {

/**
 * The check method defines the implementation of the additional matching
 * criteria to apply to a <code>ServiceItem</code> object found through 
 * standard template matching. 
 * 
 * @param item the <code>ServiceItem</code> object to test against the 
 *		additional criteria.
 *
 * @return true if the input object satisfies the additional criteria,
 * 		and false otherwise.
 * 
 */
	boolean  check(ServiceItem item);
}
